#! /usr/bin/env python
from __future__ import print_function, with_statement

import signal
import sys
from subprocess import Popen
import os
from time import sleep

def stop_process(pid):
    def stop(signum, frame):
        """
        When the signal is received, send SIGUSR1 to the provided process.
        """
        print("Sending SIGUSR1 to process %d" % (pid,))
        try:
            os.kill(pid, signal.SIGUSR1)
        except OSError:
            print("Error sending signal SIGUSR1 to %d. The process probably doesn't exist anymore." % pid, file=sys.stderr)

    return stop

def install_signal_handlers(pid):
    """
    Install a signal handler for the alarm
    """
    signal.signal(signal.SIGALRM, stop_process(pid))
    signal.signal(signal.SIGUSR1, stop_process(pid))
    signal.signal(signal.SIGUSR2, stop_process(pid))

def get_delay(value):
    """
    Interpret the delay argument.
    """
    if value[-1] in 'hsm':
        delay = float(value[:-1])
        if value[-1] == 'h':
            delay *= 3600
        elif value[-1] == 'm':
            delay *= 60
    else:
        delay = float(value)
    return int(delay)

def usage():
    print("Usage: %s delay cmd [ARGS]\n    The delay is expressed in seconds or can have the suffix 'h', 'm' or 's' for hour, minutes and seconds respectively." % sys.argv[0], file=sys.stderr)

if __name__ == "__main__":
    if len(sys.argv) < 3:
        usage()
        sys.exit(1)
    try:
        delay = get_delay(sys.argv[1])
    except ValueError:
        usage()
        sys.exit(2)
    if delay <= 0:
        print("Error, the delay must be greater than 1 second.")
        usage()
        sys.exit(2)
    try:
        cmd = Popen(sys.argv[2:])
    except OSError, err:
        print("Error, could not launch the process '%s'.\nThe error was:\n%s" % (" ".join(sys.argv[2:]), str(err)))
        sys.exit(3)
    install_signal_handlers(cmd.pid)
    signal.alarm(delay)
    try:
        result = cmd.wait()
    except OSError:
        result = cmd.wait() # If a signal is sent, the wait will fail ... so we relaunch it
    sys.exit(result)

