#! /usr/bin/python

import sys
import re

graphs = set()
main_graph = None

# Synchronize
def synchronize(data):
    return re.sub('synchronise (.*?);', r'vvgraph old\1 = \1; // Synchronize', data)

def content(data):
    data = re.sub(r'\`(.*?)\$(\w)', r'\1->old\2', data)
    data = re.sub(r'(.*?)\$', r'\1->', data)
    return data

def edge(data):
    data = re.sub(r'\(\`(.*?)\^(.*)\)\.', r'old%s.edg(\1,\2)->' % (main_graph,), data)
    data = re.sub(r'\((.*?)\^(.*)\)\.', r'%s.edge(\1,\2)->' % (main_graph,), data)
#    data = re.sub(r'\(\`(.*?[^|])\|([^\\].*)\)\.', r'old%s.arc(\1,\2)->', data)
#    data = re.sub(r'\((.*?[^|])\|([^\\].*)\)\.', r'%s.arc(\1,\2)->', data)
    data = re.sub(r'\(\`(.*?)\^(.*)\)', r'old%s.edge(\1,\2)' % (main_graph,), data)
    data = re.sub(r'\((.*?)\^(.*)\)', r'%s.edge(\1,\2)' % (main_graph,), data)
#    data = re.sub(r'\(\`(.*?[^|])\|([^\\].*)\)', r'old%s.arc(\1,\2)', data)
#    data = re.sub(r'\((.*?[^|])\|([^\\].*)\)', r'%s.arc(\1,\2)', data)
    return data

def stmt(data):
    def makesplice(m):
        return "%s.splice%s(%s,%s,%s);" % (main_graph, m.group(2).capitalize(), m.group(4), m.group(3), m.group(1))
    data = re.sub(r"splice (.*?) (after|before) (.*?) in (.*?);", makesplice, data)
    data = re.sub(r"replace (.*?) with (.*?) in (.*?);", r"%s.replace(\3,\1,\2);" % (main_graph,), data)
    def makeinsert(m):
        src = m.group(2)
        lt = m.group(1).split(',')
        result = "%s.insertEdge(%s,%s);" % (main_graph, src, lt[0])
        prev = lt[0]
        for t in lt[1:]:
            result += " %s.spliceAfter(%s,%s,%s);" % (main_graph, src, prev, t)
            prev = t
        return result
    data = re.sub(r"make {([^}]*?)} nb_of (.*?);", makeinsert, data)
    data = re.sub(r"erase (.*?) from (.*?);", r"%s.eraseEdge(\2, \1);" % (main_graph,), data)
    data = re.sub(r"remove (.*?) from (.*?);", r"\2.erase(\1);", data)
    data = re.sub(r"flag (.*?) in (.*?);", r"%s.flag(\2,\1);" % (main_graph,), data)
    return data

def forall(data):
    def replace_forall(m):
        if m.group(2).strip() in graphs:
            return "forall vertex %s in %s: {" % (m.group(1), m.group(2))
        else:
            return "forall vertex %s in %s.neighbors(%s): {" % (m.group(1), main_graph, m.group(2))
    data = re.sub(r"forall (.*?) in (.*?)\s*{", replace_forall, data)
    data = re.sub(r"forall (.*?) in \`(.*?)\s*{", r"forall vertex \1 in old%s.neighbors(\2): {" % (main_graph,), data)
    return data

def add(data):
    data = re.sub(r"add (.*?) to (.*?);", r"\2.insert(\1);", data)
    return data

def functions(data):
    data = re.sub(r"start\s*{", "init:\n{", data)
    data = re.sub(r"step\s*{", "void step() {", data)
    data = re.sub(r"render_init\s*{", "void preDraw() {", data)
    data = re.sub(r"render\s*{", "void draw() {", data)
    data = data.replace("useColour", "useColor")
    return data

graph_def = re.compile(r'mesh\s*(\w*?)\s*;')
def findGraphs(data):
    global main_graph
    global graphs
    m = graph_def.search(data)
    while m:
        if main_graph is None:
            main_graph = m.group(1)
        graphs.add(m.group(1))
        m = graph_def.search(data, m.end())

def removeSdot(data):
    for var in graphs:
        data = data.replace("%s." % (var,), "")
    data = re.sub(r"mesh\s*{((.|\n)*?)};", r"Model:\n{\1", data)
    data, cnt = re.subn(r"(vertex|edge)\s*{((.|\n)*?)};.*?(vertex|edge)\s*{((.|\n)*?)};", r"define VVGraph vvgraph:\n{\0\n}", data)
    if cnt == 0:
        data = re.sub(r"vertex\s*{((.|\n)*?)};", r"define VVGraph vvgraph:\n{\nvertex:\n{\1}\n}", data)
    data = re.sub(r"vertex\s*{((.|\n)*?)};", r"vertex:\n{\1}", data)
    data = re.sub(r"vertex (.*?)\s*=\s*0;", r"vertex \1(0);", data)
    data = re.sub(r"edge\s*{((.|\n)*?)};", r"edge:\n{\1}", data)
    data = data.replace("mesh", "vvgraph");
    return data

def includes(data):
    data = re.sub(r'(\#include .*)hpp("|>)', r'\g<1>h\2', data)
    return data

def expressions(data):
    data = re.sub(r'nextto\s*(.*?)\s*in\s*\`([^;)]*)', r'old%s.nextTo(\2,\1)' % (main_graph,), data)
    data = re.sub(r'nextto\s*(.*?)\s*in\s*([^;)]*)', r'%s.nextTo(\2,\1)' % (main_graph,), data)
    data = re.sub(r'prevto\s*(.*?)\s*in\s*\`([^;)]*)', r'old%s.prevTo(\2,\1)' % (main_graph,), data)
    data = re.sub(r'prevto\s*(.*?)\s*in\s*([^;)]*)', r'%s.prevTo(\2,\1)' % (main_graph,), data)
    data = re.sub(r'any in\s*(\b\w.*?\b)', r'%s.anyIn(\1)' % (main_graph,), data)
    data = re.sub(r'any in\s*\`(\b\w.*?\b)', r'old%s.anyIn(\1)' % (main_graph,), data)
    data = re.sub(r'empty\s*(\b\w.*?\b)', r'%s.empty(\1)' % (main_graph,), data)
    data = re.sub(r'valence\s*(\b\w.*?\b)', r'%s.valence(\1)' % (main_graph,), data)
    data = re.sub(r'empty\s*\`(\b\w.*?\b)', r'old%s.empty(\1)' % (main_graph,), data)
    data = re.sub(r'valence\s*\`(\b\w.*?\b)', r'old%s.valence(\1)' % (main_graph,), data)
    data = re.sub(r'flagged in\s*(\b\w.*?\b)', r'%s.flagged(\1)' % (main_graph,), data)
    data = re.sub(r'flagged in\s*\`(\b\w.*?\b)', r'old%s.flagged(\1)' % (main_graph,), data)
    return data

if __name__ == '__main__':
    data = open(sys.argv[1], "rt").read()
    data = data.replace('\r\n', '\n')
    data = includes(data)
    findGraphs(data)
    data = removeSdot(data)
    data = synchronize(data)
    data = forall(data)
    data = content(data)
    data = edge(data)
    data = stmt(data)
    data = add(data)
    data = expressions(data)
    data = functions(data)
    print "#include <vve.h>"
    print data
    print "}"

