#ifndef VVELIB_ALGORITHMS_DRAW_GRAPHS_H
#define VVELIB_ALGORITHMS_DRAW_GRAPHS_H

/**
 * \file algorithms/draw_graphs.h
 *
 * This file contains algorithms to draw graphs
 */

#include <config.h>
#include <util/gl.h>
#include <geometry/geometry.h>

namespace algorithms
{
  /**
   * This function draws a graph using lines between the vertices. It supposes 
   * the graph is symetric, i.e. if there is an edge from v1 to v2, there is 
   * another edge from v2 to v1.
   *
   * These methods are required in the model:
   * \code
   * Point3d position(const vertex& v);
   * Point3d normal(const vertex& v);
   * \endcode
   */
  template <typename GraphType, typename Model>
  void drawSymetricVVGraph(Model *model, const GraphType& G)
  {
    typedef typename GraphType::vertex_t vertex;
    glBegin(GL_LINES);
    forall(const vertex& v, G)
    {
      const geometry::Point3d& vpos = model->position(v);
      const geometry::Point3d& normal = model->normal(v);
      forall(const vertex& n, G.neighbors(v))
      {
        if(v<n)
        {
          glNormal3dv(normal.c_data());
          glVertex3dv(vpos.c_data());
          glNormal3dv(model->normal(n).c_data());
          glVertex3dv(model->position(n).c_data());
        }
      }
    }
    glEnd();
  }
}

#endif // VVELIB_ALGORITHMS_DRAW_GRAPHS_H

