#ifndef VVELIB_ALGORITHMS_SPLIT_H
#define VVELIB_ALGORITHMS_SPLIT_H
/**
 * \file split.h
 *
 * Defines the algorithms::split function
 */

#include <config.h>
namespace algorithms
{

  /**
   * Split the vertex \c v in two, attaching all the neighbors of \c v from \c 
   * n1 to \c n2 (included) to the new vertex. The next vertex is replacing the 
   * range n1-n2 in v.
   *
   * All the edges content are copied using the = operator. This function 
   * assumes symetric connectivity.
   */
  template <class vvgraph>
  typename vvgraph::vertex_t split(const typename vvgraph::vertex_t& v,
                                const typename vvgraph::vertex_t& n1,
                                const typename vvgraph::vertex_t& n2,
                                vvgraph& S)
  {
    typedef typename vvgraph::vertex_t vertex;
    typedef typename vvgraph::edge_t edge;
    vertex x;
    vertex pv = v;
    S.insert(x);
    S.spliceBefore(v,n1,x);
    S.insertEdge(x,v);
    do
    {
      const vertex& n = S.nextTo(v,x);
      typename edge::content_t ec = *S.edge(n,v);
      *S.replace(n,v,x) = ec;
      *S.spliceAfter(x,pv,n) = *S.edge(v,n);
      pv = n;
      S.eraseEdge(v,n);
    } while(pv != n2);
    return x;
  }
}

#endif // VVELIB_ALGORITHMS_SPLIT_H

