#ifndef VVELIB_GEOMETRY_AREA_H
#define VVELIB_GEOMETRY_AREA_H

/**
 * \file area.h
 *
 * Algorithms for areas of various shapes
 */

#include <config.h>
#include <geometry/geometry.h>
#include <vector>

namespace geometry
{
  /**
   * Area of a triangle in 2D or 3D
   */
  template <size_t N>
  double triangleArea(const util::Vector<N,double>& a, const util::Vector<N,double>& b, const util::Vector<N,double>& c)
  {
    return util::norm((a - b)^(a - c))/2.0;
  }

  /**
   * Area of a planar polygon
   */
  template <size_t N>
  double polygonArea( const std::vector<util::Vector<N,double> >& polygon )
  {
    size_t k = polygon.size(), h, l, i;
    typename util::CrossProductType<N,double>::type surface = 0;
    if(polygon.empty())
      return 0;
    h = (k-1)/2;
    if(k%2)
      l = 0;
    else
      l = k-1;
    for(i = 1 ; i < h ; i++)
      surface += (polygon[2*i] - polygon[0]) ^ (polygon[2*i+1] - polygon[2*i-1]);
    surface += (polygon[2*h] - polygon[0]) ^ (polygon[l] - polygon[2*h-1]);
    surface /= 2;
    return util::norm(surface);
  }

  /**
   * Center of mass of a 2D non-overlapping polygon
   */
  Point2d centroid( const std::vector<Point2d>& polygon);

}
#endif // VVELIB_GEOMETRY_AREA_H

