#ifndef VVELIB_GEOMETRY_GEOMETRY_H
#define VVELIB_GEOMETRY_GEOMETRY_H
/**
 * \file geometry.h
 *
 * Common definitions and utilities for all geometry algorithms
 */

#include <config.h>
#include <util/vector.h>
#include <util/matrix.h>
#include <util/floats.h>

/**
 * Algorithmic %geometry
 *
 * This namespace contains different usefull %geometry %algorithms, like 
 * intersections, areas, ...
 */
namespace geometry
{

  /**
   * Type of a 4D point
   */
  typedef util::Vector<4,double> Point4d;

  /**
   * Type of a 3D point
   */
  typedef util::Vector<3,double> Point3d;

  /**
   * Type of a 2D point
   */
  typedef util::Vector<2,double> Point2d;

  /**
   * Type of 2x2 matrix
   */
  typedef util::Matrix<2,2,double> Matrix2d;

  /**
   * Type of 3x3 matrix
   */
  typedef util::Matrix<3,3,double> Matrix3d;

  /**
   * Type of a 4x4 matrix
   */
  typedef util::Matrix<4,4,double> Matrix4d;
};

#endif // VVELIB_GEOMETRY_GEOMETRY_H

