#ifndef VVELIB_SHAPE_QUADRIC_H
#define VVELIB_SHAPE_QUADRIC_H

/**
 * \file quadric.h
 *
 * Include the definitions to draw spheres, cylinders and disks.
 */

#include <config.h>
#include <geometry/geometry.h>

/**
 * \namespace shape
 *
 * Define a set of shape to be drawn using OpenGL
 */
namespace shape
{
  using geometry::Point3d;

  /**
   * Draw a sphere
   *
   * \param center Center of the sphere
   * \param radius Radius of the sphere
   * \param slices Number of slices used to create the sphere
   * \param stacks Number of stacks used to create the sphere
   */
  void sphere(const Point3d& center, double radius, int slices, int stacks);
  /**
   * Draw a cylinder
   *
   * \param base_center Position of the center of the base of the cylinder
   * \param axis Direction of the main axis of the cylinder
   * \param radius_base Radius of the base of the cylinder
   * \param radius_top Radius at the top of the cylinder
   * \param length Length of the cylinder
   * \param slices Number of slices used to draw the cylinder
   * \param stacks Number of stacks used to draw the cylinder
   * \param closed If true, two disks are drawn at the base and top of the 
   * cylinder
   */
  void cylinder(const Point3d& base_center, const Point3d& axis,
                double radius_base, double radius_top, double length,
                int slices, int stacks,
                bool closed = true);

  /**
   * Draw a disk
   *
   * \param center Center of the disk
   * \param normal Vector normal to the surface of the disk
   * \param inner_radius Radius to start the disk, 0 for a full disk
   * \param outer_radius Radius of the disk
   * \param slices Number of slices used to draw the disk
   * \param loops Number of loops used to draw the disk
   */
  void disk(const Point3d& center, const Point3d& normal,
            double inner_radius, double outer_radius,
            int slices, int loops);
}

#endif // VVELIB_SHAPE_QUADRIC_H

