#ifndef VVELIB_STORAGE_QT_H
#define VVELIB_STORAGE_QT_H

#include <config.h>
#include <QtCore/QList>
#include <QtCore/QVector>

template <typename T>
bool serialization(storage::VVEStorage& store, QVector<T>& vec)
{
  size_t size = vec.size();
  if(!store.field("Size", size))
    return false;
  if(store.reader())
    vec.resize(size);
  for(size_t i = 0 ; i < size ; ++i)
  {
    if(!store.field("V", vec[i]))
      return false;
  }
  return true;
}

template <typename T>
bool serialization(storage::VVEStorage& store, QList<T>& vec)
{
  size_t size = vec.size();
  if(!store.field("Size", size))
    return false;
  if(store.reader())
  {
    for(size_t i = 0 ; i < size ; ++i)
    {
      T value;
      store.field("V", value);
      vec.push_back(value);
    }
  }
  else
  {
    for(typename QList<T>::iterator it = vec.begin() ; it != vec.end() ; ++it)
    {
      if(!store.field("V", *it))
        return false;
    }
  }
  return true;
}

inline bool serialization(storage::VVEStorage& store, QStringList& sl)
{
  return serialization(store, (QList<QString>&)sl);
}

template <typename T>
bool serialization(storage::VVEStorage& store, QSet<T>& vec)
{
  size_t size = vec.size();
  if(!store.field("Size", size))
    return false;
  if(store.reader())
  {
    vec.clear();
    for(size_t i = 0 ; i < size ; ++i)
    {
      T value;
      if(!store.field("V", value))
        return false;
      vec.insert(value);
    }
  }
  else
  {
    for(typename QSet<T>::iterator it = vec.begin() ; it != vec.end() ; ++it)
    {
      if(!store.field("V", *it))
        return false;
    }
  }
  return true;
}

#endif // VVELIB_STORAGE_QT_H

