#ifndef VVELIB_STORAGE_STL_H
#define VVELIB_STORAGE_STL_H

#include <config.h>
#include <vector>
#include <list>
#include <set>

namespace std
{

  template <typename T>
  bool serialization(storage::VVEStorage& store, vector<T>& vec)
  {
    size_t size = vec.size();
    if(!store.field("Size", size))
      return false;
    if(store.reader())
      vec.resize(size, T());
    for(size_t i = 0 ; i < size ; ++i)
    {
      if(!store.field("V", vec[i]))
        return false;
    }
    return true;
  }

  template <typename T, typename Sort>
  bool serialization(storage::VVEStorage& store, multiset<T, Sort>& vec)
  {
    size_t size = vec.size();
    if(!store.field("Size", size))
      return false;
    if(store.reader())
    {
      vec.clear();
      for(size_t i = 0 ; i < size ; ++i)
      {
        T value;
        if(!store.field("V", value))
          return false;
        vec.insert(value);
      }
    }
    else
    {
      for(typename multiset<T, Sort>::iterator it = vec.begin() ; it != vec.end() ; ++it)
      {
        if(!store.field("V", const_cast<T&>(*it)))
          return false;
      }
    }
    return true;
  }

  template <typename T, typename Sort>
  bool serialization(storage::VVEStorage& store, set<T, Sort>& vec)
  {
    size_t size = vec.size();
    if(!store.field("Size", size))
      return false;
    if(store.reader())
    {
      vec.clear();
      for(size_t i = 0 ; i < size ; ++i)
      {
        T value;
        if(!store.field("V", value))
          return false;
        vec.insert(value);
      }
    }
    else
    {
      for(typename set<T, Sort>::iterator it = vec.begin() ; it != vec.end() ; ++it)
      {
        if(!store.field("V", const_cast<T&>(*it)))
          return false;
      }
    }
    return true;
  }

  template <typename T>
  bool serialization(storage::VVEStorage& store, list<T>& vec)
  {
    size_t size = vec.size();
    if(!store.field("Size", size))
      return false;
    if(store.reader())
      vec.resize(size, T());
    for(typename list<T>::iterator it = vec.begin() ; it != vec.end() ; ++it)
    {
      if(!store.field("V", *it))
        return false;
    }
    return true;
  }

}

#endif // VVELIB_STORAGE_STL_H

