#ifndef VVELIB_STORAGE_TYPES_H
#define VVELIB_STORAGE_TYPES_H

/**
 * \file types.h
 *
 * This file contains all the utilities to convert froom type to type
 */

#include <config.h>
#include <QString>
#include <string>

namespace storage
{

#define NB_STORAGE_TYPES 17

  enum TYPES
  {
    T_INVALID = -1,
    T_BOOL = 0,
    T_WCHART,         // 1
    T_CHAR,           // 2
    T_SIGNED_CHAR,    // 3
    T_UNSIGNED_CHAR,  // 4
    T_SHORT,          // 5
    T_UNSIGNED_SHORT, // 6
    T_INT,            // 7
    T_UNSIGNED_INT,   // 8
    T_LONG,           // 9
    T_UNSIGNED_LONG,  // 10
    T_LONG_LONG,      // 11
    T_UNSIGNED_LONG_LONG,// 12
    T_FLOAT,          // 13
    T_DOUBLE,         // 14
    T_LONG_DOUBLE,    // 15
    T_STRING          // 16
  };

#ifndef _MSC_VER
#define FOR_ALL_TYPES(macro) \
  macro(bool) \
  macro(wchar_t) \
  macro(char) \
  macro(signed char) \
  macro(unsigned char) \
  macro(short) \
  macro(unsigned short) \
  macro(int) \
  macro(unsigned int) \
  macro(long) \
  macro(unsigned long) \
  macro(long long) \
  macro(unsigned long long) \
  macro(float) \
  macro(double) \
  macro(long double) \
  macro(std::string) \
  macro(QString)

#define FOR_ALL_TYPES_NOSTRING(macro) \
  macro(bool) \
  macro(wchar_t) \
  macro(char) \
  macro(signed char) \
  macro(unsigned char) \
  macro(short) \
  macro(unsigned short) \
  macro(int) \
  macro(unsigned int) \
  macro(long) \
  macro(unsigned long) \
  macro(long long) \
  macro(unsigned long long) \
  macro(float) \
  macro(double) \
  macro(long double)

#define FOR_ALL_TYPEIDS(macro) \
  macro(T_BOOL,bool) \
  macro(T_WCHART,wchar_t) \
  macro(T_CHAR,char) \
  macro(T_SIGNED_CHAR,signed char) \
  macro(T_UNSIGNED_CHAR,unsigned char) \
  macro(T_SHORT,short) \
  macro(T_UNSIGNED_SHORT,unsigned short) \
  macro(T_INT,int) \
  macro(T_UNSIGNED_INT,unsigned int) \
  macro(T_LONG,long) \
  macro(T_UNSIGNED_LONG,unsigned long) \
  macro(T_LONG_LONG,long long) \
  macro(T_UNSIGNED_LONG_LONG,unsigned long long) \
  macro(T_FLOAT,float) \
  macro(T_DOUBLE,double) \
  macro(T_LONG_DOUBLE,long double) \
  macro(T_STRING,QString)
#else // _MSC_VER
#define FOR_ALL_TYPES(macro) \
  macro(bool) \
  macro(char) \
  macro(signed char) \
  macro(unsigned char) \
  macro(short) \
  macro(unsigned short) \
  macro(int) \
  macro(unsigned int) \
  macro(long) \
  macro(unsigned long) \
  macro(long long) \
  macro(unsigned long long) \
  macro(float) \
  macro(double) \
  macro(long double) \
  macro(std::string) \
  macro(QString)

#define FOR_ALL_TYPES_NOSTRING(macro) \
  macro(bool) \
  macro(char) \
  macro(signed char) \
  macro(unsigned char) \
  macro(short) \
  macro(unsigned short) \
  macro(int) \
  macro(unsigned int) \
  macro(long) \
  macro(unsigned long) \
  macro(long long) \
  macro(unsigned long long) \
  macro(float) \
  macro(double) \
  macro(long double)

#define FOR_ALL_TYPEIDS(macro) \
  macro(T_BOOL,bool) \
  macro(T_CHAR,char) \
  macro(T_SIGNED_CHAR,signed char) \
  macro(T_UNSIGNED_CHAR,unsigned char) \
  macro(T_SHORT,short) \
  macro(T_UNSIGNED_SHORT,unsigned short) \
  macro(T_INT,int) \
  macro(T_UNSIGNED_INT,unsigned int) \
  macro(T_LONG,long) \
  macro(T_UNSIGNED_LONG,unsigned long) \
  macro(T_LONG_LONG,long long) \
  macro(T_UNSIGNED_LONG_LONG,unsigned long long) \
  macro(T_FLOAT,float) \
  macro(T_DOUBLE,double) \
  macro(T_LONG_DOUBLE,long double) \
  macro(T_STRING,QString)
#endif

  /**
   * Convert object of type from to object of type to.
   */
  template <typename From, typename To>
  struct ConvertType
  {
    /**
     * \returns True if the conversion was possible, false otherwise.
     */
    bool operator()(const From& from, To& to) const
    {
      to = (To)from;
      return true;
    }
  };

  template <typename T>
  struct ConvertType<T,T>
  {
    bool operator()(const T& from, T& to) const
    {
      to = from;
      return true;
    }
  };

#define CONVERT_TO_STRING(type) template <> struct ConvertType<type,QString> { bool operator()(const type& from, QString& to) const; };
#define CONVERT_FROM_STRING(type) template <> struct ConvertType<QString,type> { bool operator()(const QString&, type&) const; };
  FOR_ALL_TYPES_NOSTRING(CONVERT_TO_STRING)
  CONVERT_TO_STRING(std::string)
  FOR_ALL_TYPES_NOSTRING(CONVERT_FROM_STRING)
  CONVERT_FROM_STRING(std::string)
#undef CONVERT_TO_STRING
#undef CONVERT_FROM_STRING

#define CONVERT_TO_STRING(type) template <> struct ConvertType<type,std::string> { bool operator()(const type& from, std::string& to) const; };
#define CONVERT_FROM_STRING(type) template <> struct ConvertType<std::string,type> { bool operator()(const std::string&, type&) const; };
  FOR_ALL_TYPES_NOSTRING(CONVERT_TO_STRING)
  FOR_ALL_TYPES_NOSTRING(CONVERT_FROM_STRING)
#undef CONVERT_TO_STRING
#undef CONVERT_FROM_STRING

  /**
   * Function converting \c from to \c to.
   *
   * This allows custom (compile-time) conversion.
   */
  template <typename From, typename To>
  bool convert_type(const From& from, To& to)
  {
    static const ConvertType<From,To> converter = ConvertType<From,To>();
    return converter(from, to);
  }

  enum NUMBER_CLASS
  {
    SIGNED_INTEGER,
    UNSIGNED_INTEGER,
    FLOATING_POINT,
    CHAR,
    NOT_A_NUMBER
  };

  /**
   * Defines main traits for types directly handled
   */
  template <typename T>
  struct TypeTraits
  {
    /**
     * Name of the type.
     *
     * The name of the type is a valid C identifier
     */
    static const QString name;
    /// id of the type
    static const TYPES id;
    /**
     * Class of number.
     */
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<bool>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

#ifndef _MSC_VER
  template <> struct TypeTraits<wchar_t>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };
#endif

  template <> struct TypeTraits<char>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<signed char>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<unsigned char>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<short>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<unsigned short>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<int>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<unsigned int>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<long>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<unsigned long>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<long long>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<unsigned long long>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<float>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<double>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<long double>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<QString>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <> struct TypeTraits<std::string>
  {
    static const QString name;
    static const TYPES id;
    static const NUMBER_CLASS type;
  };

  template <typename T> const QString TypeTraits<T>::name = "invalid";
  template <typename T> const TYPES TypeTraits<T>::id = T_INVALID;
  template <typename T> const NUMBER_CLASS TypeTraits<T>::type = NOT_A_NUMBER;

  /**
   * \returns True if you can convert from \c from to \c to without loosing any 
   * information
   */
  bool isStrictConversion(TYPES from, TYPES to);

  /**
   * \returns the type id associated to the name
   */
  TYPES typename_to_id(const QString& name);

  /**
   * \returns the name associated to the type id
   */
  const QString& typeid_to_name(TYPES id);

  template <typename T>
  TYPES type_id(const T& = T())
  {
    return TypeTraits<T>::id;
  }

  template <typename T>
  const QString& type_name(const T& = T())
  {
    return TypeTraits<T>::name;
  }

  /**
   * \returns a type corresponding to the number class and size.
   *
   * If no such type exist, return T_INVALID
   */
  TYPES find_type(NUMBER_CLASS klass, size_t bytes);

}

#endif // VVELIB_STORAGE_TYPES_H

