
#line 1 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
#ifndef TEST_GRAPH_VVH
#define TEST_GRAPH_VVH

/**
 * \file graph.vvh
 *
 * File containing macros and functions to test properties on graphs.
 */

#include <test/test.h>

#include <util/forall.h>
#include <util/unorderedset.h>

/**
 * \def CHECK_NEIGHBORS_GRAPH(G)
 *
 * Check if the neighbors of the vertices of G are in G, if they are not twice 
 * in the neighborhood and if a vertex is not a neighor of itself.
 */
#define CHECK_NEIGHBORS_GRAPH(G) \
  TEST_RESULT &= test::check_neighborsGraph(G, __LINE__)

/**
 * \def CHECK_NEIGHBORS_BIGRAPH(G)
 *
 * Check if the neighbors of the vertices of G are in G, if they are not twice 
 * in the neighborhood and if a vertex is not a neighor of itself.
 */
#define CHECK_NEIGHBORS_BIGRAPH(G) \
  TEST_RESULT &= test::check_neighborsBiGraph(G, __LINE__)

/**
 * \def CHECK_SYMMETRIC_GRAPH
 *
 * Check if the graph G is symmetric and update \c result accordingly
 */
#define CHECK_SYMMETRIC_GRAPH(G) \
  TEST_RESULT &= test::check_symmetricGraph(G, __LINE__)

/**
 * \def CHECK_SYMMETRIC_GRAPH
 *
 * Check if the graph G is symmetric and update \c result accordingly
 */
#define CHECK_SYMMETRIC_BIGRAPH(G) \
  TEST_RESULT &= test::check_symmetricBiGraph(G, __LINE__)

namespace test
{

  template <typename VertexContent>
  QString toStr(const graph::Vertex<VertexContent>& v)
  {
    return QString("0x%1").arg(v.id(), 0, 16);
  }

  template <typename Graph>
  bool check_neighborsGraph(const Graph& G, size_t line_nb)
  {
    bool result = true;
    typedef typename Graph::vertex_t vertex;
    for(typeof(util::ForAll::range(G)) rangead1af6d0_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G) ; !rangead1af6d0_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead1af6d0_ca3a_11e2_85fc_0013722c2d41) {
            const vertex& v = *rangead1af6d0_ca3a_11e2_85fc_0013722c2d41.current();
#line 64 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
      std::unordered_multiset<vertex> ns;
      for(typeof(util::ForAll::range(G.neighbors(v))) rangead1c2884_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.neighbors(v)) ; !rangead1c2884_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead1c2884_ca3a_11e2_85fc_0013722c2d41) {
            const vertex& n = *rangead1c2884_ca3a_11e2_85fc_0013722c2d41.current();
#line 67 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
        if(v == n)
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex " << toStr(v) << " is its own neighbor" << endl;
          result = false;
        }
        if(!G.contains(n))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex " << toStr(n) << " is neighbor of " << toStr(v) << " but is not in the graph" << endl;
          result = false;
        }
        if(ns.count(n))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex " << toStr(n) << " is " << ns.count(n)+1 << " times the neighbor of " << toStr(v) << endl;
          result = false;
        }
        ns.insert(n);
      }}
    }}
    return result;
  }

  template <typename BiGraph>
  bool check_neighborsBiGraph(const BiGraph& G, size_t line_nb)
  {
    bool result = true;
    typedef typename BiGraph::vertex1_t vertex1;
    typedef typename BiGraph::vertex2_t vertex2;
    for(typeof(util::ForAll::range(G.vertices1())) rangead1dfcae_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.vertices1()) ; !rangead1dfcae_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead1dfcae_ca3a_11e2_85fc_0013722c2d41) {
            const vertex1& v = *rangead1dfcae_ca3a_11e2_85fc_0013722c2d41.current();
#line 99 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
      std::unordered_multiset<vertex2> ns;
      for(typeof(util::ForAll::range(G.neighbors(v))) rangead1f34d4_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.neighbors(v)) ; !rangead1f34d4_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead1f34d4_ca3a_11e2_85fc_0013722c2d41) {
            const vertex2& n = *rangead1f34d4_ca3a_11e2_85fc_0013722c2d41.current();
#line 102 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
        if(!G.contains(n))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex2 " << toStr(n) << " is neighbor of 0x"
              << QString::number(v.id(), 16) << " but is not in the graph" << endl;
          result = false;
        }
        if(ns.count(n))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex2 " << toStr(n) << " is "
              << ns.count(n)+1 << " times the neighbor of " << toStr(v) << endl;
          result = false;
        }
        ns.insert(n);
      }}
    }}
    for(typeof(util::ForAll::range(G.vertices2())) rangead1fc4bc_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.vertices2()) ; !rangead1fc4bc_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead1fc4bc_ca3a_11e2_85fc_0013722c2d41) {
            const vertex2& v = *rangead1fc4bc_ca3a_11e2_85fc_0013722c2d41.current();
#line 121 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
      std::unordered_multiset<vertex1> ns;
      for(typeof(util::ForAll::range(G.neighbors(v))) rangead1fd358_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.neighbors(v)) ; !rangead1fd358_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead1fd358_ca3a_11e2_85fc_0013722c2d41) {
            const vertex1& n = *rangead1fd358_ca3a_11e2_85fc_0013722c2d41.current();
#line 124 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
        if(!G.contains(n))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex1 " << toStr(n) << " is neighbor of 0x"
              << QString::number(v.id(), 16) << " but is not in the graph" << endl;
          result = false;
        }
        if(ns.count(n))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex1 " << toStr(n) << " is "
              << ns.count(n)+1 << " times the neighbor of " << toStr(v) << endl;
          result = false;
        }
        ns.insert(n);
      }}
    }}
    return result;
  }

  template <typename Graph>
  bool check_symmetricGraph(const Graph& G, size_t line_nb)
  {
    bool result = true;
    typedef typename Graph::vertex_t vertex;
    for(typeof(util::ForAll::range(G)) rangead202e52_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G) ; !rangead202e52_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead202e52_ca3a_11e2_85fc_0013722c2d41) {
            const vertex& v = *rangead202e52_ca3a_11e2_85fc_0013722c2d41.current();
#line 151 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
for(typeof(util::ForAll::range(G.neighbors(v))) rangead203244_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.neighbors(v)) ; !rangead203244_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead203244_ca3a_11e2_85fc_0013722c2d41) {
            const vertex& n = *rangead203244_ca3a_11e2_85fc_0013722c2d41.current();
#line 152 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
        if(!G.edge(n,v))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex " << toStr(v) << " is not a neighbor of 0x"
              << QString::number(n.id(), 16) << endl;
          result = false;
        }
      }}}
    return result;
  }

  template <typename BiGraph>
  bool check_symmetricBiGraph(const BiGraph& G, size_t line_nb)
  {
    bool result = true;
    typedef typename BiGraph::vertex1_t vertex1;
    typedef typename BiGraph::vertex2_t vertex2;
    for(typeof(util::ForAll::range(G.vertices1())) rangead206d40_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.vertices1()) ; !rangead206d40_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead206d40_ca3a_11e2_85fc_0013722c2d41) {
            const vertex1& v = *rangead206d40_ca3a_11e2_85fc_0013722c2d41.current();
#line 171 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
for(typeof(util::ForAll::range(G.neighbors(v))) rangead2070f6_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.neighbors(v)) ; !rangead2070f6_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead2070f6_ca3a_11e2_85fc_0013722c2d41) {
            const vertex2& n = *rangead2070f6_ca3a_11e2_85fc_0013722c2d41.current();
#line 172 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
        if(!G.edge(n,v))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex1 " << toStr(v) << " is not a neighbor of vertex2 0x"
              << QString::number(n.id(), 16) << endl;
          result = false;
        }
      }}}
    for(typeof(util::ForAll::range(G.vertices2())) rangead20950e_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.vertices2()) ; !rangead20950e_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead20950e_ca3a_11e2_85fc_0013722c2d41) {
            const vertex2& v = *rangead20950e_ca3a_11e2_85fc_0013722c2d41.current();
#line 182 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
for(typeof(util::ForAll::range(G.neighbors(v))) rangead2098ba_ca3a_11e2_85fc_0013722c2d41 = util::ForAll::range(G.neighbors(v)) ; !rangead2098ba_ca3a_11e2_85fc_0013722c2d41.atEnd() ; ++rangead2098ba_ca3a_11e2_85fc_0013722c2d41) {
            const vertex1& n = *rangead2098ba_ca3a_11e2_85fc_0013722c2d41.current();
#line 183 "/home/ferraro/devlp/vlab/Pascal/vve/vvelib/test/graph.vvh"
{
        if(!G.edge(n,v))
        {
          if(result) FAILED_FUNCTION(line_nb);
          out << endl << "  vertex2 " << toStr(v) << " is not a neighbor of vertex1 0x"
              << QString::number(n.id(), 16) << endl;
          result = false;
        }
      }}}
    return result;
  }

};

#endif // TEST_GRAPH_VVH

