#ifndef VVELIB_UTIL_ASSERT_H
#define VVELIB_UTIL_ASSERT_H

/**
 * \file assert.h
 *
 * Graphical (or textual) assertion utility
 *
 * Define the macro \c vvassert(expr)
 */

#include <config.h>
#include <cassert>

class QString;

namespace util
{
  void __assert_fail(const QString& assertion, const char* file,
                     unsigned int line, const char* function);
  void __assert_fail_txt(const QString& assertion, const char* file,
                         unsigned int line, const char* function);
#ifdef _MSC_VER
#  define __PRETTY_FUNCTION__ __FUNCSIG__
#endif

#ifdef NDEBUG
#  define vvassert(expr) do { expr; } while(false)
#  define vvassert_msg(expr, msg) do { expr; } while(false)
#else
#  ifndef TEXT_VVASSERT
  /**
   * \def vvassert(expr)
   *
   * If NDEBUG is not defined and expr is false, warn the user of the program 
   * of the failed assertion. If TEXT_VVASSERT is defined, vvassert falls back 
   * onto the default assert function, otherwise, it shows a Qt message box 
   * with the assert message, then abort the program.
   */
#    define vvassert(expr) ((expr) \
                            ? static_cast<void>(0) \
                            : util::__assert_fail(#expr, __FILE__, __LINE__, __PRETTY_FUNCTION__))
#    define vvassert_msg(expr, msg) ((expr) \
                            ? static_cast<void>(0) \
                            : util::__assert_fail(msg, __FILE__, __LINE__, __PRETTY_FUNCTION__))
#  else
#    define vvassert(expr) ((expr) \
                            ? static_cast<void>(0) \
                            : util::__assert_fail_txt(#expr, __FILE__, __LINE__, __PRETTY_FUNCTION__))
#    define vvassert_msg(expr, msg) ((expr) \
                            ? static_cast<void>(0) \
                            : util::__assert_fail_txt(msg, __FILE__, __LINE__, __PRETTY_FUNCTION__))
#  endif
#endif
}

#endif // VVELIB_UTIL_ASSERT_H

