#ifndef __UTIL__CONTOUR_HPP__
#define __UTIL__CONTOUR_HPP__
/**
 * \file contour.h
 *
 * Defines the util::Contour class
 */

#include <config.h>
#include <string>
#include <vector>
#include <util/vector.h>
#include <util/watchdog.h>

namespace util {
  /**
   * \class Contour contour.h <util/contour.h>
   * \brief Contour utility class.
   *
   * The Contour class encapsulates b-spline contours specified in
   * the VLAB contour formats.  Currently all versions are supported
   * (original, ver. 1 1 and 1.4).  Instances of Contour behave as
   * function objects.
   */
  class Contour : public FileObject {
  public:
    Contour();
    Contour(const Contour&);
#ifdef USE_CXX0X
    Contour(Contour&&) = default;
#endif

    Contour(std::string filename);
    Contour& operator=(const Contour&);
#ifdef USE_CXX0X
    Contour& operator=(Contour&&) = default;
#endif
    util::Vector<3,double> operator()(double t) const;
    const util::Vector<3,double>& getMax() const;
    const util::Vector<3,double>& getMin() const;

    double length(double a, double b, double dt = 0.01);
    double travel(double t, double l, double dt = 0.01);

    util::Vector<3,double> tangent(double t, double dt = 0.01);
    util::Vector<3,double> normal(double t, double dt = 0.01);

    void reread();

  private:
    double Basis0(double t) const;
    double Basis1(double t) const;
    double Basis2(double t) const;
    double Basis3(double t) const;

    std::vector<util::Vector<3,double> > pts;
    util::Vector<3,double> max;
    util::Vector<3,double> min;

    bool closed;
    bool regular;
  };
}

#endif
