#ifndef UTIL_FLOATS_H
#define UTIL_FLOATS_H

namespace util
{
#ifndef VVE_REL_EPSILON
  /**
   * \def VVE_REL_EPSILON
   * Epsilon value used for relative comparisons
   *
   * \relates geometry
   */
#define VVE_REL_EPSILON 1e-8
#endif

#ifndef VVE_EPSILON
  /**
   * \def VVE_EPSILON
   * Epsilon value used for absolute comparisons
   *
   * \relates geometry
   */
#define VVE_EPSILON 1e-9
#endif

  /**
   * Test if v1 > v2, with floating point error
   */
  inline bool greater(double v1, double v2, double abs_epsilon = VVE_EPSILON, double rel_epsilon = VVE_REL_EPSILON)
  {
    return (v1*(1+rel_epsilon/2) + abs_epsilon/2) > (v2*(1-rel_epsilon/2) - abs_epsilon/2);
  }

  /**
   * Test if v1 < v2, with floating point error
   */
  inline bool less(double v1, double v2, double abs_epsilon = VVE_EPSILON, double rel_epsilon = VVE_REL_EPSILON)
  {
    return greater(v2, v1, abs_epsilon, rel_epsilon);
  }

  /**
   * Test if v1 == v2, with floating point error
   */
  inline bool approx(double v1, double v2, double abs_epsilon = VVE_EPSILON, double rel_epsilon = VVE_REL_EPSILON)
  {
    double diff = fabs(v1-v2);
    if(diff < abs_epsilon) return true;
    double tot = fabs(v1)+fabs(v2);
    return (diff/tot) < rel_epsilon;
  }

  /**
   * Test if v1 != v2, with floating point error
   */
  inline bool notequal(double v1, double v2, double abs_epsilon = VVE_EPSILON, double rel_epsilon = VVE_REL_EPSILON)
  {
    return !approx(v1, v2, abs_epsilon, rel_epsilon);
  }

}

#endif // UTIL_FLOATS_H

