#ifndef KEY_FRAMER_H
#define KEY_FRAMER_H
#include <config.h>
#include <qgl.h>
#include <qstring.h>
#include <util/watchdog.h>
#include "bsurface.h"

namespace util
{

  using Shapes::BSplineSurface;

  struct Frame{
    BSplineSurface key;
    char *name;
    double time;
    double scale;
  };

  /**
   * \class KeyFramer key_framer.h <util/key_framer.h>
   *
   * Class describing the evolution of a bspline surface through time.
   *
   * The description consists in a list of `BSplineSurface` objects at given 
   * time. Intermediate time are computed using linear interpolation between 
   * key frames.
   *
   */
  class KeyFramer : public util::FileObject {

  public:

    KeyFramer();
    KeyFramer(const KeyFramer&);
    ~KeyFramer();

    bool Load(const char* fname);
    bool Save(const char* fname);

    void reread()
    {
      std::string fn = getFilename();
      Load(fn.c_str());
    }

    void SetParent(QGLWidget *parent)
    {gl_context=parent;}

    int FindFrame(double time);
    double FindTime(unsigned int frame);
    void SetTime(double time);
    void SetFrame(unsigned int frame);
    double GetTime()
    {return sel_time;}
    unsigned int GetFrame()
    {return sel_frame;};

    void AddFrame();
    void DeleteFrame();
    void LoadSurface(const char* fname);

    void DisplayTime(double time);
    void DisplayFrame(unsigned int frame);
    void DisplayTimeline();

    //Not implemented
    void SetBoundingBox(Point2d min, Point2d max);
    void SetTime(Point3d pos);
    void SelectClosest(Point3d pos);
    void MoveSelect(Point3d pos);
    void AdvanceTime(double time_increment);
    void SetScale(double scale)
    { if(scale>0)keys[sel_frame].scale=scale;}
    void SetPrecision(int u,int v)
    { u_prec=u;v_prec=v;};
    BSplineSurface GetSurfaceFrame(unsigned int frame);
    BSplineSurface GetSurfaceFrame();
    BSplineSurface GetSurfaceTime(double time);
    BSplineSurface GetSurfaceTime();

  private:
    int u_prec; int v_prec;
    int max_u,max_v;
    QGLWidget *gl_context;
    Point2d min;
    Point2d max;
    std::vector< Frame > keys;
    unsigned int sel_frame;
    double sel_time;
    double time_interval[2];
  };

} // namespace util
#endif 
