#ifndef __VVPVIEWER_HPP__
#define __VVPVIEWER_HPP__

#include <view/orthoviewer.hpp>
#include <string>
#include <set>
#ifdef FAM_THREAD
#include <qtimer.h>
#include "fam.hpp"
class QCloseEvent;

// Minimum time between two writing event before sending it of to the viewer 
// (in msec)
#define EVENT_TIMER 10

#endif


class DllInterface;

class VVPApp;

class VVPViewer : public view::OrthoViewer {
  Q_OBJECT

public:
  VVPViewer(DllInterface *dll = 0, QWidget* parent = 0, const char* name = 0, Qt::WFlags f = 0);

  ~VVPViewer();
  void setInterface(DllInterface* pInterface);

  inline void renderText(int x, int y, QString s, QFont f = QFont()) {
    QGLWidget::renderText(x, y, s, f);
  }
  inline void renderText(double x, double y, double z, QString s, QFont f = QFont()) {
    QGLWidget::renderText(x, y, z, s, f);
  }

#ifdef FAM_THREAD
  void registerFile( std::string filename );
  void unregisterFile( std::string filename );
  void reregisterFiles();
#endif

protected:
  virtual void initializeGL();
  virtual void paintGL();
  virtual void resizeGL(int w, int h);
#ifdef FAM_THREAD
  void customEvent( QCustomEvent* event );
#endif

private:
  DllInterface* dll_interface;
  VVPApp*       vvpapp;

#ifdef FAM_THREAD
  FamThread *fam_thread;
  QTimer timer;
  std::set<std::string> modified_files;
  std::map<std::string,std::set<std::string> > absolute_to_user_filenames;
#endif

private slots:
  void send_start();
  void send_step();
  void send_end();
  void send_run();
  void send_animate();
  void send_stop();
  void send_reread();
  void send_quit();

signals:
  void start();
  void step();
  void end();
  void run();
  void animate();
  void stop();
  void reread( const std::set<std::string>& filenames );
  void quit();

public:
  void buildContextMenu();
};

#endif
