// Surface class
#ifndef __SURFACE_H__
#define __SURFACE_H__

#include <string>
#include <cmath>
#include <util/parms.hpp>
#include <util/point.hpp>

using std::string;

const int SAMPLES = 10000;	// Sample count for function/contours

const int ARCLENGTH = 0;	// Growth types
const int CONTOUR = 1;

const double PI = 3.14159265358979323846;
const double PIx2 = PI + PI;   // 2 PI

const double DX = .00015;

class SurfacePoint {
  public:
    SurfacePoint() {};
    ~SurfacePoint() {};
    double ArcLength();
    double Angle();
    util::Point<double> Pos();
    util::Point<double> Normal();
    friend std::ostream& operator<<(std::ostream& os, const SurfacePoint &sp) {
      return os;
     };
    friend std::istream& operator>>(std::istream& is, SurfacePoint &sp) {
      return is;
    };
    friend class Surface;
  private:
    util::Point<double> pos;		// x,y,z pos of vertex
    util::Point<double> normal;		// normalized normal vector
    double arcLength;		// Arclength of vertex
    double angle;		// Angle
};

class Surface {
  public: 
    Surface(util::Parms &parms, string section);
    ~Surface();

    void Zero(SurfacePoint &p);
    bool SetPoint(SurfacePoint &p, SurfacePoint sp, util::Point<double> cp);
    void Grow(SurfacePoint &p, double dt);
    double Distance(SurfacePoint &u, SurfacePoint &v);

  private:
    void LoadFunction(string funcFile, double *v);
    void IntegrateFunction(string funcFile, double *v);
    void ArcLengthParam(string contFile, util::Point<double> *v);
    void CalcPos(SurfacePoint &p);
    void CalcNormal(SurfacePoint &p);

    int growthType;

    double dArc;                  // arclength diff between points in posV
    util::Point<double> dPos;     // amount to add for next point extrapolate
    util::Point<double>arcLV[SAMPLES + 1]; // Pos vec, param by arclength
    double growthVA[SAMPLES + 1]; // Growth vector, param by arclength
    double surfaceScale;	  // Contour scaling constant
    double growthScale;		  // Growth scaling constant
    string arcLengthGrowthFile;	  // Arc length growth function file name
    string surfaceContourFile;	  // Surface Contour file name
    int rootSearchMaxSteps;	// Max steps for closest point on surface
};

#endif
