// Surface class
#ifndef __SURFACE_H__
#define __SURFACE_H__

#include <string>
#include <cmath>
#include <util/parms.hpp>
#include <util/point.hpp>

#include "bezier.h"

using std::string;

const int SAMPLES = 10000;	// Sample count for function/contours

const int ARCLENGTH = 0;	// Growth types
const int CONTOUR = 1;

const double PI = 3.14159265358979323846;
const double PIx2 = PI + PI;   // 2 PI

const double DX = .00000001;
const double MAXSEARCHSTEPS = 500;
const int MAXSURF = 10;

class SurfacePoint {
  public:
    SurfacePoint() {};
    ~SurfacePoint() {};

    util::Point<double> Pos();
    util::Point<double> Normal();

    friend std::ostream& operator<<(std::ostream& os, const SurfacePoint &sp) {
      return os;
     };
    friend std::istream& operator>>(std::istream& is, SurfacePoint &sp) {
      return is;
    };
    friend class Surface;

  private:
    double u,v;                         // Coordinates in parametric space
    util::Point<double> pos;		// x,y,z pos of vertex
    util::Point<double> normal;		// normalized normal vector
};

class Surface {
  public: 
    Surface(util::Parms &parms, string section);
    ~Surface() {};

    // Create zero point
    void Zero(SurfacePoint &p);

    // Create inital point
    void InitPoint(SurfacePoint &p, double u, double v);
   
    // Find closest point on surface to a given point with an initial guess
    bool SetPoint(SurfacePoint &p, SurfacePoint sp, util::Point<double> cp);

    // Advance growth by dt
    void GrowStep(double dt);

    // Compute pos of point based on current time
    void GetPos(SurfacePoint &p);

    // Distance between two surface points
    double Distance(SurfacePoint &u, SurfacePoint &v);

    friend class SurfacePoint;

  private:
    void CalcPos(SurfacePoint &p);      // Calc x,y,x pos from u,v values
    void CalcNormal(SurfacePoint &p);   // Calc normal

    double growthScale;			// Growth scaling constant

    int surfaces;                       // Surface for growth stages
    double surfMaxDist;                 // Max dist for closest point search
    double surfTimeScale;               // Surface time scale
    Bezier surface[MAXSURF];            // Bezier surfaces
    double surfScale[MAXSURF];          // Surface scaling constants
    double surfTime[MAXSURF];           // Surdace time scale
    Bezier surfCurr;                    // Current surfaces

    double time;                        // Time
};

#endif
