[Main]
PrintInterval: 20
RandomSeed: 0
DrawInterval: 1

[Chemicals]
DiffA: 0.015   // Activator diffusion
mumax: 0.38    // Activator decay
mumin: 0.08
rhomax: 0.38   // Activator production
rhomin: 0.08
rhorand: .15   // Variation on rho (between 0 and 1)
kappa: 0.5     // Activator saturation
rho0: 0.01     // Activator base production

DiffS: 0.05    // Substrate diffusion
nu: 0          // Substrate decay
sigmamax: 0.19 // Substrate production
sigmamin: 0.04

[Seashell] // Sea chell configuration
NbCells: 2000      // Number of cells in a line
CellWidth: .2      // Width of a cell
CellHeight: .6     // Height of a cell (i.e. time between two lines of cell)
CellDeath: 1      // Time for a cell to die
Circle: false

// Position of the control points to initialize the parameters
// The curve is defined as:
// |      |      |      |      |
// |    __|______|__    |      |
// |   /  |      |  \   |      |
// |__/   |      |   \__|______|
// |______|______|______|______|__
// |      |      |      |      |
// 0      1      2      3      4 - index in the InitCurve vector
InitCurve: 30 120 150 240
InitCurveWidth: 90 // Total width of the curve (it the seashell is wider, the curve is repeated)

[View]
ColorLow: 0    // Color of the minimum concentration
ColorHigh: 1   // Color of the maximum concentration
MaxView: 4    // Concentration corresponding to ColorHigh
TextureChunk: 200 // Number of pixel of a texture chunk
Background: 0.0 // Grey level of the background

[Solver]
Solver: AdaptiveCrankNicholson		// Euler, AdaptiveEuler, Fixedpoint, 
                                        // Midpoint, RungeKutta, 
                                        // AdaptiveRungeKutta, 
                                        // AdaptiveCrankNicholson

// Global help:
//  *TolType can be MeanComponent or MaxComponent

// Various Timesteps
EulerDt: .05//.0025			// Timestep for Euler solver
FixedPointDt: .06			// Timestep for fixed point solver
MidPointDt: .06				// Timestep for midpoint solver
RungeKuttaDt: .08			// Timestep for Runge-Kutta 4th order
InitialDt: .1				// Beginning timestep for adaptive solvers
MinDt: .001
MaxDt: 25				// Maximum Dt

// Adaptive Euler parms
AEulerIncDt: .1				// Adaptive Euler Dt increment/decrement
AEulerResDt: .5				// Adaptive Euler restart Dt decrement
AEulerTolType: MaxComponent		// Tolerence type for adaptive Euler
AEulerResTol: 1				// Adaptive Euler restart tolerance
AEulerLowTol: .3			// Adaptive Euler low water mark
AEulerHighTol: .4			// Adaptive Euler high water mark

// Fixed Point iteration parms
FixedPointMaxSteps: 5		// Max steps for fixed point iteration
FixedPointTol: .0001			// Tolerance for fixed point iteration
FixedPointTolType: MaxComponent

// Adaptive Runge-Kutta parms
ARungeIncDt: .01			// Adaptive Runge-Kutta Dt increment/decrement
ARungeResDt: .5				// Adaptive Runge-Kutta restart Dt decrement
ARungeTolType: MaxComponent		// Tolerence type for adaptive Runge-Kutta 
ARungeResTol: 1				// Adaptive Runge-Kutta restart tolerance
ARungeLowTol: .03			// Adaptive Runge-Kutta low water mark
ARungeHighTol: .05			// Adaptive Runge-Kutta high water mark

// Crank Nicholson params
CRIncDt: .2				// Increment for timestep based on efficiency (mulitple of current size)
CRResDt: .5				// Decrement for timestep if unsucc
CRAvgCPU: .5                            // Weight of current dt in eff average
CRMinCPU: 5				// Minimun CPU, below this alway increases timestep

// Newton Tolerance (used by Crank Nicholson)
NewtTol: .01				// Tolerence for Newton's method
NewtTolType: MeanComponent
NewtMaxSteps: 10 			// Max steps for Newton's method

// Conjugate gradient params (used by Crank Nicholson)
ConjGradTol: .001			// Tolerence for conjugate gradient
ConjGradTolType: MeanComponent
ConjGradMaxSteps: .05			// Max steps for conjugate gradient (multiple of N)
ConstNbPartials: true			// Set if partials to neighbors are constant (diffusion only)

// Misc general parms
Dx: .0001				// Delta for numerical diff
PrintMatrix: false			// Print Matrix (Conj-Grad)
PrintStats: 0

