#include <util/vector.h>

#define NB_CHEMICALS 2

typedef util::Vector<4,double> Point4d;

#define EXTRA_CONTENT \
  double sigma;  /* Substrate production in the cell*/

#include "seashell_model.cpp"

class NautilusPompiliusModel : public SeashellModel
{
public:
  double diffA;              // Activator diffusion
  double mu;                 // Activator decay
  double rho;                // Activator production
  double kappa;              // Activator saturation
  double rho0;               // Activator base production
  double diffS;              // Substrate diffusion
  double nu;                 // Substrate decay
  double sigmamax, sigmamin; // Substrate production

  NautilusPompiliusModel(QObject *parent, bool do_init = true)
    : SeashellModel(parent)
  {
    if(do_init)
      init();
  }

  void readRDParams( util::Parms& parms )
  {
    // Read chemicals parameters
    parms("Chemicals", "DiffA", diffA);
    parms("Chemicals", "mu", mu);
    parms("Chemicals", "rho", rho);
    parms("Chemicals", "kappa", kappa);
    parms("Chemicals", "rho0", rho0);
    parms("Chemicals", "DiffS", diffS);
    parms("Chemicals", "nu", nu);
    parms("Chemicals", "sigmamax", sigmamax);
    parms("Chemicals", "sigmamin", sigmamin);
  }

  void initCell(vertex v)
  {
    int i = v->pos;
    double x = double(i) / double(nbCells);
    v->sigma = x*(sigmamax - sigmamin) + sigmamin;
  }

  void updateDerivatives(const vertex& v, const rd_tag_t&)
  {
    double A = v->chems[0];
    double S = v->chems[1];
    double AxA = A*A;
    double AProd = rho*S*(AxA/(1+kappa*AxA) + rho0);

    double dA = AProd - mu*A;
    double dS = v->sigma - AProd - nu*S;

    // Diffusion
    forall(const vertex& n, LiveCells.neighbors(v))
    {
      dA += diffA*(n->chems[0] - A)/cellWidth;
      dS += diffS*(n->chems[1] - S)/cellWidth;
    }

    v->derivs[0] = dA;
    v->derivs[1] = dS;
  }

};

//DEFINE_MODEL(NautilusPompiliusModel)

